/*`
 * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
 */

#include "e_mod_main.h"
#include "e_mod_config.h"

/* module private routines */
static Eloquence *_eloquence_new();
static void _eloquence_shutdown(Eloquence *eloquence);
static void _eloquence_config_menu_new(Eloquence *eloquence);

static Eloquence_Face *_eloquence_face_new(E_Container *con);
static void _eloquence_face_free(Eloquence_Face *face);
static void _eloquence_face_enable(Eloquence_Face *face);
static void _eloquence_face_disable(Eloquence_Face *face);
static void  _eloquence_face_menu_new(Eloquence_Face *face);

static void _eloquence_face_cb_menu_configure(Eloquence_Face *face, 
		E_Menu *m, E_Menu_Item *mi);
static void _eloquence_face_cb_gmc_change(Eloquence_Face *face, 
		E_Gadman_Client *gmc, E_Gadman_Change  change);
static void _eloquence_face_cb_mouse_down(Eloquence_Face *face, 
		Evas *e, Evas_Object *obj, void *event_info);
static void _eloquence_face_cb_menu_edit(Eloquence_Face *face, 
		E_Menu *m, E_Menu_Item *mi);
static void _eloquence_face_cb_update_policy(Eloquence_Face *face);

static void _eloquence_face_cb_pre_menu_control(Eloquence_Face *face, E_Menu *menu);
static void _eloquence_face_cb_post_menu_control(Eloquence_Face *face, E_Menu *menu);
static void _control_menu_load(E_Menu *menu, Eloquence_Face *face);

static int _eloquence_face_switch_theme(Eloquence_Face *face, const char *theme);
static void _eloquence_face_switch_player(Eloquence_Face *face);

static void callback_player_play_pause(Eloquence_Face *face, Evas_Object *obj, const char *emission, 
					const char *source);
static void callback_player_track_next(Eloquence_Face *face, Evas_Object *obj, const char *emission, 
					const char *source);
static void callback_player_track_prev(Eloquence_Face *face, Evas_Object *obj, const char *emission, 
					const char *source);
static void callback_player_vol_inc(Eloquence_Face *face, Evas_Object *obj, const char *emission, 
					const char *source);
static void callback_player_vol_dec(Eloquence_Face *face, Evas_Object *obj, const char *emission, 
					const char *source);
static void callback_player_shuffle(Eloquence_Face *face, Evas_Object *obj, const char *emission, 
					const char *source);
static void callback_gui_menu(Eloquence_Face *face, Evas_Object *obj, const char *emission, 
					const char *source);

/* Module specific private routines */
static int _eloquence_count;

static E_Config_DD *conf_edd;
static E_Config_DD *conf_face_edd;


/* public module routines. all modules must have these */
E_Module_Api e_modapi =
{
	   E_MODULE_API_VERSION,
	      "Eloquence"
};
	
void *
e_modapi_init(E_Module *module)
{
   Eloquence *eloquence;
  
   /* check module api version */
   if (module->api->version < E_MODULE_API_VERSION) {
	e_error_dialog_show(_("Module API Error"),
			    _("Error initializing Module: Eloquence\n"
			      "It requires a minimum module API version of: %i.\n"
			      "The module API advertized by Enlightenment is: %i.\n"
			      "Aborting module."),
			    E_MODULE_API_VERSION,
			    module->api->version);
	return NULL;
     }

   /* actually init eloquence */
   eloquence = _eloquence_new();
   module->config_menu = eloquence->config_menu;
   return eloquence;
}

int
e_modapi_shutdown(E_Module *module)
{
   Eloquence *eloquence;

   if (module->config_menu)
     module->config_menu = NULL;

   eloquence = module->data;
   if (eloquence){
	   if (eloquence->config_dialog){
		   e_object_del(E_OBJECT(eloquence->config_dialog));
		   eloquence->config_dialog = NULL;
	   }
	   _eloquence_shutdown(eloquence);
   }

   return 1;
}

int
e_modapi_save(E_Module *module)
{
   Eloquence *eloquence;

   eloquence = module->data;
   e_config_domain_save("module.eloquence", conf_edd, eloquence->conf);
   return 1;
}

int
e_modapi_info(E_Module *module)
{
   char buf[4096];

   snprintf(buf, sizeof(buf), "%s/module_icon.png", e_module_dir_get(module));
   module->icon_file = strdup(buf);
   return 1;
}

int
e_modapi_about(E_Module *module)
{
   e_module_dialog_show(_("Enlightenment Eloquence Module"),
		       _("Shows the title of the song playing in various media players."));
   return 1;
}

int
e_modapi_config(E_Module *m)
{
	Eloquence *eloquence;
	Eloquence_Face *face;

	eloquence = m->data;
	if (!eloquence) return 0;
	if (!eloquence->faces) return 0;
	face = eloquence->faces->data;
	if (!face) return 0;
	_config_eloquence_module(face->con, face->eloquence);
	return 1;
}

void _eloquence_face_cb_config_updated(Eloquence_Face *face) {
	_eloquence_face_switch_theme(face, face->conf->theme);
	_eloquence_face_switch_player(face);
	_eloquence_face_cb_update_policy(face);
}


/* module private routines */
static Eloquence *
_eloquence_new()
{
   Eloquence *eloquence;
   Evas_List *managers, *l, *l2, *cl;
   E_Menu_Item *mi;

   _eloquence_count = 0;
   eloquence = E_NEW(Eloquence, 1);
   if (!eloquence) return NULL;

   conf_face_edd = E_CONFIG_DD_NEW("Eloquence_Config_Face", Config_Face);
#undef T
#undef D
#define T Config_Face
#define D conf_face_edd
   E_CONFIG_VAL(D, T, enabled, UCHAR);
   E_CONFIG_VAL(D, T, allow_overlap, INT);
   E_CONFIG_VAL(D, T, theme, STR);
   E_CONFIG_VAL(D, T, player, INT);
   E_CONFIG_VAL(D, T, poll_time, DOUBLE);
   E_CONFIG_VAL(D, T, input_charset, STR);

   conf_edd = E_CONFIG_DD_NEW("Eloquence_Config", Config);
#undef T
#undef D
#define T Config
#define D conf_edd
   E_CONFIG_LIST(D, T, faces, conf_face_edd);

   eloquence->conf = e_config_domain_load("module.eloquence", conf_edd);
   if (!eloquence->conf) {
	   eloquence->conf = E_NEW(Config, 1);
   }
   
   _eloquence_config_menu_new(eloquence);

   managers = e_manager_list();
   cl = eloquence->conf->faces;
   for (l = managers; l; l = l->next) {
	E_Manager *man;

	man = l->data;
	for (l2 = man->containers; l2; l2 = l2->next) {
	     E_Container *con;
	     Eloquence_Face *face;

	     con = l2->data;
	     face = _eloquence_face_new(con);
	     if (face) {
		     face->eloquence = eloquence;
		     eloquence->faces = evas_list_append(eloquence->faces, face);
		     if (!cl) {
			     face->conf = E_NEW(Config_Face, 1);
			     face->conf->enabled = 1;
			     face->conf->allow_overlap = 0;
			     face->conf->poll_time = 1.0;
			     face->conf->theme = evas_stringshare_add(THEME_DEFAULT);
#ifdef HAVE_BMP_XMMSCTRL_H
			     face->conf->player = PLAYER_XMMS;
#endif
#ifdef HAVE_XMMSCLIENT_XMMSCLIENT_H
			     face->conf->player = PLAYER_XMMS2;
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
			     face->conf->player = PLAYER_BMP;
#endif
#ifdef HAVE_BMPX_METADATA_H
			     face->conf->player = PLAYER_BMPX;
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
			     face->conf->player = PLAYER_AUDACIOUS;
#endif
#ifdef HAVE_LIBMPD_LIBMPDCLIENT_H
			     face->conf->player = PLAYER_MPD;
#endif
#ifdef HAVE_AMAROK
			     face->conf->player = PLAYER_AMAROK;
#endif
			     face->conf->input_charset = evas_stringshare_add(CHARSET_DEFAULT);
			     eloquence->conf->faces = evas_list_append(
					     eloquence->conf->faces, face->conf);
		     }
		     else {
			     face->conf = cl->data;
			     cl = cl->next;
		     }
		     E_CONFIG_LIMIT(face->conf->poll_time, 0.1, 10.0);
		     E_CONFIG_LIMIT(face->conf->allow_overlap, 0, 1);

		     _eloquence_face_menu_new(face);
		     mi = e_menu_item_new(eloquence->config_menu);
		     e_menu_item_label_set(mi, con->name);
		     e_menu_item_submenu_set(mi, face->menu);

		     _eloquence_face_switch_theme(face, face->conf->theme);
		     _eloquence_face_switch_player(face);
	     }
	}
   }
   return eloquence;
}

static void
_eloquence_shutdown(Eloquence *eloquence)
{
   Evas_List *list;

   E_CONFIG_DD_FREE(conf_edd);
   E_CONFIG_DD_FREE(conf_face_edd);

   for (list = eloquence->faces; list; list = list->next){
	   Eloquence_Face *face = list->data;
	   ecore_timer_del(face->update_timer);
#ifdef HAVE_AMAROK
           if (face->conf->player == PLAYER_AMAROK)
		   amarok_free(face);
#endif
	   _eloquence_face_free(list->data);
   }
   evas_list_free(eloquence->faces);

   e_object_del(E_OBJECT(eloquence->config_menu));


     
   evas_list_free(eloquence->conf->faces);
   free(eloquence->conf);
   free(eloquence);
}

static void
_eloquence_config_menu_new(Eloquence *eloquence)
{
   eloquence->config_menu = e_menu_new();
}

static Eloquence_Face *
_eloquence_face_new(E_Container *con)
{
   Eloquence_Face *face;
   Evas_Object *o;

   face = E_NEW(Eloquence_Face, 1);
   if (!face) return NULL;

   e_object_ref(E_OBJECT(con));
   face->con = con;

   return face;
}

static void
_eloquence_face_free(Eloquence_Face *face)
{
   if (face->update_timer)
	   ecore_timer_del(face->update_timer);

   e_object_unref(E_OBJECT(face->con));

   if (face->gmc)
      e_object_del(E_OBJECT(face->gmc));
   if (face->eloquence_object)
      evas_object_del(face->eloquence_object);
   if (face->event_object)
      evas_object_del(face->event_object);

   e_object_del(E_OBJECT(face->menu));

   if (face->conf->theme)
	   evas_stringshare_del(face->conf->theme);
   
   free(face->conf);
   free(face);

   _eloquence_count--;
}

static void
_eloquence_face_enable(Eloquence_Face *face)
{
   face->conf->enabled = 1;

   if (face->eloquence_object)
      evas_object_show(face->eloquence_object);
   if (face->event_object)
      evas_object_show(face->event_object);

   e_config_save_queue();
}

static void
_eloquence_face_disable(Eloquence_Face *face)
{
   face->conf->enabled = 0;

   if (face->eloquence_object)
      evas_object_hide(face->eloquence_object);
   if (face->event_object)
      evas_object_hide(face->event_object);

   e_config_save_queue();
}

static void
_eloquence_face_menu_new(Eloquence_Face *face) {
   E_Menu *mn, *smn;
   E_Menu_Item *mi;
   mn = e_menu_new();
   face->menu = mn;

   mi = e_menu_item_new(mn);
   e_menu_item_label_set(mi, _("Configuration"));
   e_menu_item_callback_set(mi, _eloquence_face_cb_menu_configure, face);

   mi = e_menu_item_new(mn);
   e_menu_item_label_set(mi, _("Edit Mode"));
   e_menu_item_callback_set(mi, _eloquence_face_cb_menu_edit, face);

   smn = e_menu_new();
   e_menu_pre_activate_callback_set(smn, _eloquence_face_cb_pre_menu_control, face);
   e_menu_post_deactivate_callback_set(smn, _eloquence_face_cb_post_menu_control, face);
   
   face->control_menu = smn;
   mi = e_menu_item_new(mn);
   e_menu_item_label_set(mi, _("Control Player"));
   e_menu_item_submenu_set(mi, face->control_menu);
}

static void
_eloquence_face_cb_menu_configure(Eloquence_Face *face, E_Menu *m, E_Menu_Item *mi) {
  if (!face) return;
  _config_eloquence_module(face->con, face->eloquence);
}

static void
_eloquence_face_cb_gmc_change(Eloquence_Face *face, E_Gadman_Client *gmc, E_Gadman_Change change)
{
   Evas_Coord x, y, w, h;

   switch (change)
     {
      case E_GADMAN_CHANGE_MOVE_RESIZE:
	 e_gadman_client_geometry_get(face->gmc, &x, &y, &w, &h);
	 evas_object_move(face->eloquence_object, x, y);
	 evas_object_move(face->event_object, x, y);
	 evas_object_resize(face->eloquence_object, w, h);
	 evas_object_resize(face->event_object, w, h);
	 break;
      case E_GADMAN_CHANGE_RAISE:
	 evas_object_raise(face->eloquence_object);
	 evas_object_raise(face->event_object);
	 break;
      case E_GADMAN_CHANGE_EDGE:
      case E_GADMAN_CHANGE_ZONE:
	 /* FIXME
	  * Must we do something here?
	  */
	 break;
     }
}

static void
_eloquence_face_cb_mouse_down(Eloquence_Face *face, Evas *e, Evas_Object *obj, void *event_info) {
	/* We just don't use it here and catch all mouse-clicks with the edje */	
}

static void
_eloquence_face_cb_menu_edit(Eloquence_Face *face, E_Menu *m, E_Menu_Item *mi) {
   e_gadman_mode_set(face->gmc->gadman, E_GADMAN_MODE_EDIT);
}

static void _eloquence_face_cb_update_policy(Eloquence_Face *face) {
  E_Gadman_Policy policy;

  policy = face->gmc->policy;
  
  if (face->conf->allow_overlap == 0)
	  policy &= ~E_GADMAN_POLICY_ALLOW_OVERLAP;
  else
	  policy |= E_GADMAN_POLICY_ALLOW_OVERLAP;
  e_gadman_client_policy_set(face->gmc , policy);
}

	

void draw_text(Eloquence_Face *face){
  size_t insize = (strlen(face->song))+1;
  size_t avail = insize+2;

   face->song_utf8 = face->outbuf;
   size_t iconv_errorcode = iconv(face->cd, &face->song, &insize, &face->song_utf8, &avail);
   edje_object_part_text_set(face->eloquence_object, "title", face->outbuf);
}

static void _eloquence_face_cb_pre_menu_control(Eloquence_Face *face, E_Menu *menu){
  if (menu->realized || !face)
	  return;
  // generate menu
  _control_menu_load(menu, face);
}

static void _eloquence_face_cb_post_menu_control(Eloquence_Face *face, E_Menu *menu){
  if (menu->parent_item && menu->parent_item->active)
	  return;
  // remove all menuitems
  Evas_List *items;
  for (items = menu->items; items; items = items->next)
	  e_object_del(E_OBJECT(items->data));
  evas_list_free(menu->items);
  menu->items = NULL;
}

static void _control_menu_load(E_Menu *menu, Eloquence_Face *face){
   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
	 xmms_init_menu(menu, face);
         break;	      
#endif
#ifdef HAVE_XMMSCLIENT_XMMSCLIENT_H
	 case PLAYER_XMMS2:
	 xmms2_init_menu(menu, face);
	 break;
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
	 bmp_init_menu(menu, face);
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
	 bmpx_init_menu(menu, face);
         break;	      
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
	 audacious_init_menu(menu, face);
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
	 amarok_init_menu(menu, face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_LIBMPDCLIENT_H
      case PLAYER_MPD:
	 mpd_connect(face);
	 mpd_init_menu(menu, face);
         break;	      
#endif
     }
}

static int
_eloquence_face_switch_theme(Eloquence_Face *face, const char *theme){
   Evas_Object *o;
   E_Gadman_Policy  policy;

   face->cd = iconv_open ("UTF-8", face->conf->input_charset);

   char path[PATH_MAX] = PACKAGE_DATA_DIR"/themes/";
   strcat(strcat(path, theme), ".edj");

   evas_event_freeze(face->con->bg_evas);
   o = edje_object_add(face->con->bg_evas);
   
   if ( edje_object_file_set(o, path, "eloquence/main") )
   {
	   if (face->eloquence_object)
		   evas_object_del(face->eloquence_object);

	   // don't free if old and new are the same piece of memory (like on startup)
	   if (face->conf->theme != theme){
		   if (face->conf->theme)
			   evas_stringshare_del(face->conf->theme);
		   face->conf->theme = evas_stringshare_add(theme);
	   }

	   face->eloquence_object = o;

	   edje_object_signal_callback_add(o, "pause", "pause", callback_player_play_pause, face);
	   edje_object_signal_callback_add(o, "next", "next", callback_player_track_next, face);
	   edje_object_signal_callback_add(o, "prev", "prev", callback_player_track_prev, face);
	   edje_object_signal_callback_add(o, "inc_vol", "inc_vol", callback_player_vol_inc, face);
	   edje_object_signal_callback_add(o, "dec_vol", "dec_vol", callback_player_vol_dec, face);
	   edje_object_signal_callback_add(o, "shuffle", "shuffle", callback_player_shuffle, face);
           edje_object_signal_callback_add(o, "options", "options", callback_gui_menu, face);
	   
	   if (face->conf->enabled)
		   evas_object_show(o);
	   if (!face->event_object){ // no need to recreate
		   o = evas_object_rectangle_add(face->con->bg_evas);
		   face->event_object = o;
		   evas_object_layer_set(o, 2);
		   evas_object_repeat_events_set(o, 1);
		   evas_object_color_set(o, 0, 0, 0, 0);

		   if (face->conf->enabled)
			   evas_object_show(o);
	   }
	   
	   int w, h;
	   edje_object_size_min_get(o, &w, &h);
	   
	   // avoid reloading gadman to keep size/position
	   if (!face->gmc){
		   face->gmc = e_gadman_client_new(face->con->gadman);
		   e_gadman_client_domain_set(face->gmc, 
				   "module.eloquence", _eloquence_count++);
		   policy = E_GADMAN_POLICY_ANYWHERE |
			    E_GADMAN_POLICY_HMOVE |
			    E_GADMAN_POLICY_VMOVE |
			    E_GADMAN_POLICY_HSIZE |
			    E_GADMAN_POLICY_VSIZE;
		   if (face->conf->allow_overlap == 0)
			   policy &= ~E_GADMAN_POLICY_ALLOW_OVERLAP;
		   else
			   policy |= E_GADMAN_POLICY_ALLOW_OVERLAP;
		   e_gadman_client_policy_set(face->gmc, policy);
		   e_gadman_client_align_set(face->gmc, 1.0, 1.0);
		   e_gadman_client_resize(face->gmc, 640, 30); // set default size
		   e_gadman_client_change_func_set(face->gmc, 
				   _eloquence_face_cb_gmc_change, face);
		   e_gadman_client_load(face->gmc);
	   }
	   
	   e_gadman_client_auto_size_set(face->gmc, w, h);
	   edje_object_size_max_get(o, &w, &h);
	   e_gadman_client_max_size_set(face->gmc, w, h);

	   e_config_save_queue();
   }
   else{  // unable to open theme, keep current
	   evas_object_del(o);
	   o = NULL;
   }

   evas_event_thaw(face->con->bg_evas);

   return o != NULL;
}

static void
_eloquence_face_switch_player(Eloquence_Face *face){
   if(face->update_timer)
	     ecore_timer_del(face->update_timer);

#ifdef HAVE_LIBMPD_LIBMPDCLIENT_H
   if (face->old_player == PLAYER_MPD)
	   mpd_disconnect(face);
#endif
#ifdef HAVE_AMAROK
   if (face->old_player == PLAYER_AMAROK)
	   amarok_free(face);
#endif

   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
         face->update_timer = ecore_timer_add(
			 face->conf->poll_time, 
			 xmms_update_song, face);
         break;	      
#endif
#ifdef HAVE_XMMSCLIENT_XMMSCLIENT_H
      case PLAYER_XMMS2:
	 xmms2_connect(face);
	 face->update_timer = ecore_timer_add(
			 face->conf->poll_time, 
			 xmms2_update_song, face);
         break;
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
         face->update_timer = ecore_timer_add(
			 face->conf->poll_time, 
			 bmp_update_song, face);
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
         face->update_timer = ecore_timer_add(
			 face->conf->poll_time, 
			 bmpx_update_song, face);
         break;	      
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
         face->update_timer = ecore_timer_add(
			 face->conf->poll_time, 
			 audacious_update_song, face);
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
	 amarok_init(face);
	 face->update_timer = ecore_timer_add(
			 face->conf->poll_time, 
			 amarok_update_song, face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_LIBMPDCLIENT_H
      case PLAYER_MPD:
	 mpd_connect(face);
	 face->update_timer = ecore_timer_add(
			 face->conf->poll_time, 
			 mpd_update_song, face);
         break;	      
#endif
     }
}

static void callback_player_play_pause(Eloquence_Face *face, 
		Evas_Object *obj, const char *emission, const char *source){
   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
         xmms_control_play_pause(face);
         break;	      
#endif
#ifdef HAVE_XMMSCLIENT_XMMSCLIENT_H
      case PLAYER_XMMS2:
	 xmms2_control_play_pause(face);
         break;              
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
         bmp_control_play_pause(face);
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
         bmpx_control_play_pause(face);
         break;	      
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
         audacious_control_play_pause(face);
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
         amarok_control_play_pause(face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_LIBMPDCLIENT_H
      case PLAYER_MPD:
	 mpd_control_play_pause(face);
         break;              
#endif
     }

}

static void callback_player_track_next(Eloquence_Face *face, 
		Evas_Object *obj, const char *emission, const char *source){
   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
         xmms_control_track_next(face);
         break;	      
#endif
#ifdef HAVE_XMMSCLIENT_XMMSCLIENT_H
      case PLAYER_XMMS2:
	 xmms2_control_track_next(face);
         break;              
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
         bmp_control_track_next(face);
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
         bmpx_control_track_next(face);
         break;	      
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
         audacious_control_track_next(face);
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
         amarok_control_track_next(face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_LIBMPDCLIENT_H
      case PLAYER_MPD:
	 mpd_control_track_next(face);
         break;              
#endif
     }
	
}

static void callback_player_track_prev(Eloquence_Face *face, 
		Evas_Object *obj, const char *emission, const char *source){
   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
         xmms_control_track_prev(face);
         break;	      
#endif
#ifdef HAVE_XMMSCLIENT_XMMSCLIENT_H
      case PLAYER_XMMS2:
	 xmms2_control_track_prev(face);
         break;              
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
         bmp_control_track_prev(face);
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
         bmpx_control_track_prev(face);
         break;	      
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
         audacious_control_track_prev(face);
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
         amarok_control_track_prev(face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_LIBMPDCLIENT_H
      case PLAYER_MPD:
	 mpd_control_track_prev(face);
         break;              
#endif
     }
	
}

static void callback_player_vol_inc(Eloquence_Face *face, 
		Evas_Object *obj, const char *emission, const char *source){
   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
         xmms_control_vol_inc(face);
         break;	      
#endif
#ifdef HAVE_XMMSCLIENT_XMMSCLIENT_H
      case PLAYER_XMMS2:
	 /* xmms2 doesn't yet support this */
         break;              
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
         bmp_control_vol_inc(face);
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
         bmpx_control_vol_inc(face);
         break;	      
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
         audacious_control_vol_inc(face);
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
         amarok_control_vol_inc(face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_LIBMPDCLIENT_H
      case PLAYER_MPD:
	 mpd_control_vol_inc(face);
         break;              
#endif
     }

}

static void callback_player_vol_dec(Eloquence_Face *face, 
		Evas_Object *obj, const char *emission, const char *source){
   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
         xmms_control_vol_dec(face);
         break;	      
#endif
#ifdef HAVE_XMMSCLIENT_XMMSCLIENT_H
      case PLAYER_XMMS2:
	 /* xmms2 doesn't yet support this */
         break;              
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
         bmp_control_vol_dec(face);
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
         bmpx_control_vol_dec(face);
         break;	      
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
         audacious_control_vol_dec(face);
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
         amarok_control_vol_dec(face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_LIBMPDCLIENT_H
      case PLAYER_MPD:
	 mpd_control_vol_dec(face);
         break;              
#endif
     }
	
}

static void callback_player_shuffle(Eloquence_Face *face, 
		Evas_Object *obj, const char *emission, const char *source){
   switch (face->conf->player)
     {
#ifdef HAVE_XMMS_XMMSCTRL_H
      case PLAYER_XMMS:
         xmms_control_shuffle(face);
         break;	      
#endif
#ifdef HAVE_BMP_BEEPCTRL_H
      case PLAYER_BMP:
/*  disabled because crashing bmp, bad bug in bmp!!!
	bmp_control_shuffle(face);
*/
         break;	      
#endif
#ifdef HAVE_BMPX_METADATA_H
      case PLAYER_BMPX:
	bmpx_control_shuffle(face);
         break;	      
#endif
#ifdef HAVE_XMMSCLIENT_XMMSCLIENT_H
      case PLAYER_XMMS2:
	 xmms2_control_shuffle(face);
         break;
#endif
#ifdef HAVE_AUDACIOUS_BEEPCTRL_H
      case PLAYER_AUDACIOUS:
/*  disabled because crashing Audacious, bad bug in Audacious!!!
         audacious_control_shuffle(face);
*/
         break;	      
#endif
#ifdef HAVE_AMAROK
      case PLAYER_AMAROK:
         amarok_control_shuffle(face);
         break;	      
#endif
#ifdef HAVE_LIBMPD_LIBMPDCLIENT_H
      case PLAYER_MPD:
	 mpd_control_shuffle(face);
         break;              
#endif
     }

}

static void callback_gui_menu(Eloquence_Face *face, 
		Evas_Object *obj, const char *emission, const char *source){
   E_Manager *man;
   E_Container *con;
   int x,y;
   
   man = e_manager_current_get();
   if (!man) return;
   con = e_container_current_get(man);
   if (!con) return;

   e_gadman_client_geometry_get(face->gmc, &x, &y, NULL, NULL);
   
   e_menu_activate_mouse(face->menu, e_zone_current_get(con),
		   x, y, 1, 1, E_MENU_POP_DIRECTION_DOWN, NULL);
   e_util_container_fake_mouse_up_all_later(con);
}


